from wata.pointcloud.utils import utils
from wata.pointcloud.utils import move_pcd
from wata.pointcloud.utils import o3d_visualize_utils
from wata.pointcloud.pcd_vis import PointCloudVisualize
from wata.pointcloud.pcd_tanway import PointCloudTanway
from numpy import ndarray
from pathlib import Path
from typing import Union



class PointCloudProcess(PointCloudVisualize,PointCloudTanway):

    @staticmethod
    def cut_pcd(points: ndarray, pcd_range: list, axis: str):
        '''
        **功能描述**: 切割指定范围的点云  
        
        Args:  
            points: numpy格式的点云  
            pcd_range: [x1,y1,z1,x2,y2,z2]  
            axis: 可选 "xy","xz","yz","xz","xyz",默认为"xyz"  

        Returns:  
            返回范围内的点  
        '''
        return utils.cut_pcd(points, pcd_range, axis)

    @staticmethod
    def filter_points(points: ndarray, del_points: ndarray):
        '''
        **功能描述**: 过滤删除部分点,要求del_points是points的子集  
        
        Args:  
            points: numpy格式的点云  
            del_points: 要从points中删除的点,numpy格式的点云  

        Returns:  
            剩余的点,numpy格式  
        '''
        return utils.filter_points(points, del_points)

    @staticmethod
    def points_to_o3d_model(points: ndarray, point_colors: Union[None, list] = None):
        '''
        **功能描述**: 将numpy格式的点云转化为open3d模型  
        
        Args:
            points: numpy格式的点云  
            point_colors: 点云的颜色信息,可以为None,可以为长度为3的元素为int类型的RGB列表  

        Returns:  
            open3d模型  
        '''
        return o3d_visualize_utils.points_to_o3d_model(points=points, point_colors=point_colors)

    @staticmethod
    def save_o3d_camera_parameters(points: ndarray, save_file_path: Union[str, Path] = 'camera_parameters.json',
                                   window_size: list = [1200, 800]):
        '''
        **功能描述**:  保存手动调整的open3d的视角  
        
        Args:  
            points: numpy格式的点云  
            save_file_path: open3d 可视化视角文件保存的路径 json  
            window_size: 可视化窗口的大小 list  

        Returns:  
            无  
        '''
        return o3d_visualize_utils.save_o3d_camera_parameters(points, save_file_path, window_size)

    @staticmethod
    def get_points(path: str, num_features: Union[None, int] = None):
        '''
        **功能描述**: 读取点云文件获取点
        
        Args:
            path: 点云文件的路径支持 pcd npy bin 格式的点云  
            num_features: 当点云文件格式为pcd 或npy时,num_features为加载点的列数; 当点云文件格式为bin时,num_features为bin文件点云的列数  

        Returns:  
            返回numpy格式的点云矩阵  
        '''
        return utils.get_points(path, num_features)

    @staticmethod
    def pcd2bin(pcd_dir: Union[str, Path], bin_dir: Union[str, Path], num_features=4):
        '''
        **功能描述**: pcd格式点云转bin格式  
        
        Args:  
            pcd_dir: pcd格式点云数据的存放目录  
            bin_dir: bin格式点云的数据村帆帆目录  
            num_features: bin格式的点云列数  

        Returns:  
            无  
        '''
        utils.pcd2bin(pcd_dir, bin_dir, num_features)

    @staticmethod
    def xyzrpy2RTmatrix(xyz_rpy, seq="xyz", degrees=True):
        '''
        **功能描述**: xyzrpy2RTmatrix  
        
        Args:
            xyz_rpy: list [dx, dy, dz, roll, pitch, yaw]  
            seq: 绕轴旋转的顺序  
            degrees: roll, pitch, yaw 是弧度还是角度 默认为弧度(False)  

        Returns:  
            numpy格式的4x4旋转平移矩阵   
        '''
        return move_pcd.xyzrpy2RTmatrix(xyz_rpy, seq, degrees)

    @staticmethod
    def RTmatrix2xyzrpy(RTmatrix: ndarray,seq="xyz", degrees=True):
        '''
        **功能描述**: RTmatrix2xyzrpy  
        
        Args:  
            RTmatrix: numpy格式的4x4旋转平移矩阵
            seq: 绕轴旋转的顺序  
            degrees: 保存角度值还是弧度值,默认True

        Returns:  
            numpy格式1x6矩阵 分别为 dx, dy, dz, roll, pitch, yaw  
        '''
        return move_pcd.RTmatrix2xyzrpy(RTmatrix,seq, degrees)

    @staticmethod
    def move_pcd_with_RTmatrix(points: ndarray, RTmatrix: ndarray, inv=False):
        '''
        **功能描述**: 通过旋转平移矩阵移动点云  
        
        Args:
            points: numpy格式的点云  
            RTmatrix: numpy格式的4x4旋转平移矩阵  
            inv: 是否对 RTmatrix 取逆,默认为False  

        Returns:  
            旋转平移后的numpy格式的点云  
        '''
        return move_pcd.move_pcd_with_RTmatrix(points, RTmatrix, inv)

    @staticmethod
    def move_pcd_with_xyzrpy(points: ndarray, xyz_rpy, seq="xyz", degrees=False):
        '''
        **功能描述**: 用 dx, dy, dz, roll, pitch, yaw 旋转平移点云,注意按照xyz的顺序先旋转再平移
        
        Args:  
            points: numpy格式的点云,需要有x,y,z维度  
            xyz_rpy: 列表格式[dx, dy, dz, roll, pitch, yaw]  
            degrees: xyz_rpy 中的roll, pitch, yaw是否为角度值,默认为False  

        Returns:  
            返回旋转平移后的点云  
        '''
        return move_pcd.move_pcd_with_xyzrpy(points, xyz_rpy, seq, degrees=degrees)
    
    @staticmethod
    def rotate_pointcloud(points: ndarray,rpy, seq="xyz",degrees=True):
        '''
        **功能描述**: 用 roll, pitch, yaw 旋转点云,根据seq选择旋转顺序
        
        Args:  
            points: numpy格式的点云,需要有x,y,z维度  
            rpy: 列表格式[roll, pitch, yaw]  
            degrees: rpy 中的roll, pitch, yaw是否为角度值,默认为True  

        Returns:  
            返回旋转后的点云  
        '''
        return move_pcd.rotate_pointcloud(points,rpy,seq,degrees)
    
    @staticmethod
    def translate_pointcloud(points: ndarray,xyz):
        '''
        **功能描述**: 用 x y z 平移点云  

        Args:  
            points: numpy格式的点云,需要有x,y,z维度  
            xyz: 列表格式[dx, dy, dz]  

        Returns:  
            返回平移后的点云  
        '''
        return move_pcd.translate_pointcloud(points, xyz)

    @staticmethod
    def cartesian_to_spherical(points: ndarray, degrees=False):
        '''
        **功能描述**: 直角坐标系转极坐标系,单位是弧度
        
        Args:  
            points: 直角坐标系点云  
            degrees: 返回值是否为角度,默认False 弧度  

        Returns:
            (r, theta竖直极角, phi水平方位角)的numpy格式点云  
        '''
        return utils.cartesian_to_spherical(points=points, degrees=degrees)

    @staticmethod
    def get_v_channel_from_pcd(points: ndarray, vfov, channel_nums, offset=0.01):
        '''
        **功能描述**: 获取垂直方向的通道  
        
        Args:  
            points: numpy格式的点云,需要有x,y,z维度  
            vfov: 长度为2的列表,代表了垂直视场角的范围,水平方向为0度,低于水平方向为负  
            channel_nums: 垂直通道的线数  
            offset: 偏移量  

        Returns:  
            numpy格式的 nx1 的数组  
        '''
        return utils.get_v_channel_from_pcd(points, vfov, channel_nums, offset)

    @staticmethod
    def get_h_channel_from_pcd(points: ndarray, hfov, channel_nums, offset=0.001):
        '''
        **功能描述**: 获取点云水平方向的通道  
        
        Args:
            points: numpy格式的点云,需要有x,y,z维度  
            hfov: 长度为2的列表,代表了水平视场角的范围,前向120度的视场角可用[30,150]表示  
            channel_nums: 水平通道的线数  
            offset: 偏移量  

        Returns:
            numpy格式的 nx1 的数组  
        '''
        return utils.get_h_channel_from_pcd(points, hfov, channel_nums, offset)

    @staticmethod
    def points_in_boxes(points: ndarray, boxes, type="gpu"):
        '''
        **功能描述**: 此API需要安装pytorch,用于标记和查找点云中在指定框内的点  
        
        Args:
            points: numpy格式的点云,需要有x,y,z维度  
            boxes: nx7的numpy矩阵  
            type: 可选"gpu" "cpu"  

        Returns:
            返回每个点所在框的索引列表  
        '''
        return utils.points_in_boxes(points, boxes, type)

    @staticmethod
    def save_pcd(points: ndarray, save_path, fields=None, npdtype=None, type='binary_compressed'):
        '''
        **功能描述**: 保存pcd格式的点云  
        
        Args:
            points: numpy格式的点云  
            save_path: pcd格式的点云文件保存的路径  
            fields: 点云每一列的元信息,当为None时默认为["x","y","z","4","5",....]  
            npdtype: 点云每一列保存的数据类型,支持numpy格式和字符串格式。字符串格式可选 "f32","f64","i8","i16","i32","i64","u8","u16","u32","u64"  
            type: 有三种格式,分别为“binary”, “ascii”,“binary_compressed” ,默认为“binary_compressed”。binary格式保存为二进制文件,ascii保存为普通文件,“binary_compressed”保存为二进制压缩格式

        Returns:  
            无  
        '''
        return utils.save_pcd(points, save_path, fields, npdtype, type)

    @staticmethod
    def array_to_structured_array(array, fields, npdtype):
        '''
        **功能描述**: 将普通的numpy转换为结构化的numpy数据

        Args:
            array: numpy格式的矩阵
            fields: 矩阵每一列的元信息
            npdtype: 点云每一列保存的数据类型。字符串格式可选 "f32","f64","i8","i16","i32","i64","u8","u16","u32","u64"

        Returns:
            返回结构化数据
        '''
        return utils.array_to_structured_array(array, fields, npdtype)

    @staticmethod
    def extract_bev_points_by_mask(points, mask, range, other_index=-1):
        '''
        **功能描述**: 根据提供的mask提取对应部分的点云

        Args:
            points:numpy数据
            mask:numpy格式的mask
            range:mask作用的点云的范围,如[xmin,ymin,xmax,ymax]
        Returns:
            返回每个点的mask索引列表
        '''
        return utils.extract_bev_points_by_mask(points, mask, range, other_index)

    @staticmethod
    def structured_array_to_array(structured_array):
        '''
        **功能描述**: 将结构化的numpy转换为普通矩阵

        Args:
            structured_array：结构化numpy数据

        Returns:
            返回普通numpy矩阵
        '''
        return utils.structured_array_to_array(structured_array)



from wata.pointcloud.utils import utils
from numpy import ndarray
import numpy as np
from pathlib import Path
from typing import Union




class PointCloudTanway:

    @staticmethod
    def saveTanwayRoadPCDBinaryCompressed(points: ndarray, save_path):
        '''
        **功能描述**: 保存 BinaryCompressed pcd格式的点云,仅支持tanway 路端的数据
        
        Args:
            points: numpy格式的点云  
            save_path: pcd格式的点云文件保存的路径

        Returns:  
            无  
        '''
        fields = ['x', 'y', 'z', 'intensity', 'channel', 'angle', 'echo', 'mirror', 'block', 't_sec', 't_usec', 'lidar_id']
        npdtype = ['f32', 'f32', 'f32', 'f32', 'i32','f32', 'i32', 'i32', 'i32', 'u32', 'u32', 'i32']

        utils.save_pcd(points, save_path=save_path, fields=fields, npdtype=npdtype, type='binary_compressed')


    @staticmethod
    def get_anno_from_tanwayjson(json_data):
        boxes = []
        class_list = []
        for agent in json_data:
            boxes.append(
                [agent["position3d"]["x"], agent["position3d"]["y"], agent["position3d"]["z"], agent["size3d"]["x"],
                 agent["size3d"]["y"], agent["size3d"]["z"], agent["heading"]])
            class_list.append(agent["type"])
        return np.array(boxes), class_list


import numpy as np
from pathlib import Path
import os
import tqdm
import glob
import struct

import wata
# from wata import obtain_cur_path_cmd
from wata.file.utils import utils as file
from wata.pointcloud.utils.load_pcd import get_points_from_pcd_file
from wata.pointcloud.utils.load_ldo import decode_lidar as get_points_from_cognata_ldo_file
from wata.pointcloud.utils.o3d_visualize_utils import open3d_draw_scenes, show_pcd_from_points_by_open3d
from wata.pointcloud.utils.qtopengl_visualize_utils import show_pcd_from_points_by_qtopengl
from wata.pointcloud.utils.plot_visualize_utils import plot_draw_scenes, show_pcd_from_points_by_matplotlib
from wata.file.utils.type_mapping import numpy_type_to_pcd_type, np_type_to_numpy_type, numpy_type_to_struct_type


def cut_pcd(points, pcd_range, axis='xyz'):
    if axis == "x":
        x_range = [pcd_range[0], pcd_range[1]]
        mask = (x_range[0] <= points[:, 0]) & (points[:, 0] <= x_range[1])
    elif axis == "y":
        y_range = [pcd_range[0], pcd_range[1]]
        mask = (y_range[0] <= points[:, 1]) & (points[:, 1] <= y_range[1])
    elif axis == "z":
        z_range = [pcd_range[0], pcd_range[1]]
        mask = (z_range[0] <= points[:, 2]) & (points[:, 2] <= z_range[1])
    elif axis == "xy" or "yx":
        x_range = [pcd_range[0], pcd_range[2]]
        y_range = [pcd_range[1], pcd_range[3]]
        mask = (x_range[0] <= points[:, 0]) & (points[:, 0] <= x_range[1]) & (
                y_range[0] <= points[:, 1]) & (points[:, 1] <= y_range[1])
    elif axis == "xz" or "zx":
        x_range = [pcd_range[0], pcd_range[2]]
        z_range = [pcd_range[1], pcd_range[3]]
        mask = (x_range[0] <= points[:, 0]) & (points[:, 0] <= x_range[1]) & (
                z_range[0] <= points[:, 2]) & (points[:, 2] <= z_range[1])
    elif axis == "yz" or "zy":
        y_range = [pcd_range[0], pcd_range[2]]
        z_range = [pcd_range[1], pcd_range[3]]
        mask = (y_range[0] <= points[:, 1]) & (points[:, 1] <= y_range[1]) & (
                z_range[0] <= points[:, 2]) & (points[:, 2] <= z_range[1])
    elif axis == "xyz" or "xzy" or "yxz" or "yzx" or "zxy" or "zyx":
        x_range = [pcd_range[0], pcd_range[3]]
        y_range = [pcd_range[1], pcd_range[4]]
        z_range = [pcd_range[2], pcd_range[5]]
        mask = (x_range[0] <= points[:, 0]) & (points[:, 0] <= x_range[1]) & (
                y_range[0] <= points[:, 1]) & (points[:, 1] <= y_range[1]) & (
                z_range[0] <= points[:, 2]) & (points[:, 2] <= z_range[1])
    else:
        raise ValueError("axis format error, only supports any combination in the x, y, and z directions")
    points = points[mask]
    return points


def filter_points(points, del_points):
    pcd1_set = set(map(tuple, points))
    pcd2_set = set(map(tuple, del_points))
    result_set = pcd1_set - pcd2_set
    result = np.array(list(result_set))
    return result


def get_points(path, num_features):
    pcd_ext = Path(path).suffix
    if pcd_ext == '.bin':
        num_features = 4 if num_features is None else num_features
        points = np.fromfile(path, dtype=np.float32).reshape(-1, num_features)
    elif pcd_ext == ".npy":
        points = np.load(path)
        points = points[:, 0:num_features]
    elif pcd_ext == ".pcd":
        points = get_points_from_pcd_file(path, num_features=num_features)
    elif pcd_ext in [".ldo", ".ldg", ".ldx"]: # only cognata sim data
        points = get_points_from_cognata_ldo_file(path)
    else:
        raise NameError("Unable to handle {} formatted files".format(pcd_ext))
    return points


def pcd2bin(pcd_dir, bin_dir, num_features=4):
    file.mkdir_if_not_exist(bin_dir)
    pcd_list = glob.glob(pcd_dir + "./*.pcd")
    for pcd_path in tqdm.tqdm(pcd_list):
        filename, _ = os.path.splitext(pcd_path)
        filename = filename.split("\\")[-1]
        points = get_points_from_pcd_file(pcd_path, num_features=num_features)
        points = points[:, 0:num_features].astype(np.float32)
        bin_file = os.path.join(bin_dir, filename) + '.bin'
        points.tofile(bin_file)
    print("==> The bin file has been saved in \"{}\"".format(bin_dir))


def show_pcd(path, point_size=1, background_color=None, pcd_range=None, bin_num_features=None, create_coordinate=True,
             create_plane=True, type='open3d'):
    points = get_points(path, num_features=bin_num_features)
    if pcd_range:
        points = cut_pcd(points, pcd_range)
    show_pcd_from_points(points=points, point_size=point_size, background_color=background_color,
                         create_coordinate=create_coordinate, create_plane=create_plane,
                         type=type)


def show_pcd_from_points(points, point_size=1, background_color=None, colors=None, create_coordinate=True,
                         create_plane=True, type='open3d', savepath=None, plot_range=None, o3d_cam_param=None,
                         o3d_window_size=[1200, 800]):
    if type == 'open3d':
        show_pcd_from_points_by_open3d(
            points=points, point_size=point_size,
            background_color=background_color,
            create_coordinate=create_coordinate,
            create_plane=create_plane,
            colors=colors,
            cam_param=o3d_cam_param,
            window_size=o3d_window_size
        )
    elif type == 'qtopengl':
        show_pcd_from_points_by_qtopengl(
            points=points,
            point_size=point_size,
            background_color=background_color,
            create_coordinate=create_coordinate,
            create_plane=create_plane
        )
    elif type == 'matplotlib':
        show_pcd_from_points_by_matplotlib(
            points=points,
            point_size=point_size,
            background_color=background_color,
            colors=colors,
            create_coordinate=create_coordinate,
            savepath=savepath, plot_range=plot_range
        )
    elif type == 'mayavi':
        pass
    elif type == 'vispy':
        pass


def add_boxes(points, gt_boxes=None, gt_labels=None, pred_boxes=None, pred_labels=None, pred_scores=None, point_size=1,
              background_color=None, create_plane=True, point_colors=None, create_coordinate=True, type='open3d',
              savepath=None, plot_range=None, o3d_cam_param=None, o3d_window_size=[1200, 800]):
    if type == 'open3d':
        open3d_draw_scenes(points=points, gt_boxes=gt_boxes, gt_labels=gt_labels,
                           pred_boxes=pred_boxes, pred_labels=pred_labels, pred_scores=pred_scores,
                           point_size=point_size, background_color=background_color, create_plane=create_plane,
                           create_coordinate=create_coordinate, cam_param=o3d_cam_param,
                           window_size=o3d_window_size)
    elif type == 'qtopengl':
        pass
    elif type == 'matplotlib':
        plot_draw_scenes(points=points, gt_boxes=gt_boxes, gt_labels=gt_labels,
                         pred_boxes=pred_boxes, pred_labels=pred_labels, pred_scores=pred_scores,
                         point_size=point_size, background_color=background_color,
                         point_colors=point_colors,
                         create_coordinate=create_coordinate, savepath=savepath, plot_range=plot_range)
    elif type == 'mayavi':
        pass
    elif type == 'vispy':
        pass


def cartesian_to_spherical(points, degrees=False):
    points_cloud = points.copy()
    x = points_cloud[:, 0]
    y = points_cloud[:, 1]
    z = points_cloud[:, 2]
    r = np.sqrt(x ** 2 + y ** 2 + z ** 2)
    dis = np.sqrt(x ** 2 + y ** 2)

    theta = np.arctan2(z, dis)  # 极角
    phi = np.arctan2(y, x)  # 方位角
    if degrees:
        theta = np.rad2deg(theta)  # 极角
        phi = np.rad2deg(phi)  # 方位角
    spherical_points = np.column_stack((r, theta, phi))
    points[:, 0:3] = spherical_points[:, 0:3]
    return points


def get_v_channel_from_pcd(points, vfov, channel_nums, offset=0.01):
    x = points[:, 0]
    y = points[:, 1]
    z = points[:, 2]
    theta = np.rad2deg(np.arctan2(z, np.sqrt(x ** 2 + y ** 2)))  # 极角
    v_angle = vfov[1] - vfov[0] + 2 * offset
    v_resolution = v_angle / channel_nums
    v_channel = ((vfov[1] + offset - theta) / v_resolution + 1).astype(int)
    return v_channel


def get_h_channel_from_pcd(points, hfov, channel_nums, offset=0.001):
    x = points[:, 0]
    y = points[:, 1]
    z = points[:, 2]
    # theta = np.rad2deg(np.arctan2(z, np.sqrt(x ** 2 + y ** 2)))  # 极角
    phi = np.rad2deg(np.arctan2(x, y))
    if min(phi) < hfov[0]:
        hfov[0] = min(phi)

    h_angle = hfov[1] - hfov[0] + 2 * offset
    h_resolution = h_angle / channel_nums
    h_channel = ((hfov[1] + offset - phi) / h_resolution + 1).astype(int)
    return h_channel


def points_in_boxes(points, boxes, type="gpu"):
    import torch
    from wata.pointcloud.ops.roiaware_pool3d import roiaware_pool3d_utils

    if isinstance(points, np.ndarray):
        if type == "gpu":
            points = torch.from_numpy(points[:, :3]).unsqueeze(dim=0).float().cuda()
        else:
            points = torch.from_numpy(points[:, :3]).unsqueeze(dim=0).float().cpu()
    if isinstance(boxes, np.ndarray):
        if type == "gpu":
            boxes = torch.from_numpy(boxes).unsqueeze(dim=0).float().cuda()
        else:
            boxes = torch.from_numpy(boxes).unsqueeze(dim=0).float().cpu()

    if type == "gpu":
        box_idxs_of_pts = roiaware_pool3d_utils.points_in_boxes_gpu(points, boxes).cpu().numpy()
    else:
        box_idxs_of_pts = roiaware_pool3d_utils.points_in_boxes_cpu(points, boxes).numpy()
    return box_idxs_of_pts


def array_to_structured_array(array, fields, npdtype):
    dylist = []
    for i in range(len(fields)):
        dylist.append((fields[i], npdtype[i]))
    dtype = np.dtype(dylist)
    structured_array = np.zeros(array.shape[0], dtype=dtype)
    for i, field in enumerate(fields):
        structured_array[field] = array[:, i].astype(npdtype[i])
    return structured_array

def structured_array_to_array(structured_array):
    points = structured_array[structured_array.dtype.names[0]].reshape(-1, 1)
    for fieldname in structured_array.dtype.names[1:]:
        points_column = np.ascontiguousarray(structured_array[fieldname])
        points = np.hstack((points, points_column.reshape(-1, 1)))
    return points


def save_pcd(points, save_path, fields=None, npdtype=None, type='binary_compressed'):
    npdtype_is_exist = True
    point_num, fields_num = points.shape[0], points.shape[1]
    if npdtype is None:
        npdtype_is_exist = False
        points = points.astype(np.float32)
        npdtype = [np.dtype('float32') for _ in range(fields_num)]
    else:
        assert len(
            npdtype) == fields_num, "The length of <npdtype> should be consistent with the number of columns in the pcd."

    if isinstance(npdtype[0], str):
        npdtype = [np_type_to_numpy_type(npdtype[i]) for i in range(fields_num)]

    one_point = points[0, :]
    TYPE, SIZE = [], []
    struct_formats = ""
    for i in range(len(one_point)):
        struct_formats += numpy_type_to_struct_type(npdtype[i])
        type_size = numpy_type_to_pcd_type(npdtype[i])
        TYPE.append(type_size[0])
        SIZE.append(str(type_size[1]))
    TYPE = " ".join(TYPE)
    SIZE = " ".join(SIZE)
    COUNT = " ".join(map(str, np.ones(int(fields_num), dtype=np.int8)))

    if fields is None:
        fields_str = " ".join(map(str, np.arange(4, fields_num + 1)))  # 生成从4到n的数组
        fields_str = "x y z " + fields_str
    else:
        assert len(
            fields) == fields_num, "The length of <fields> should be consistent with the number of columns in the pcd."
        fields_str = ' '.join(fields)

    pcd_file = open(save_path, 'wb')

    pcd_file.write('# .PCD v0.7 - Point Cloud Data file format\n'.encode('utf-8'))
    pcd_file.write('VERSION 0.7\n'.encode('utf-8'))

    pcd_file.write(f'FIELDS {fields_str}\n'.encode('utf-8'))
    pcd_file.write(f'SIZE {SIZE}\n'.encode('utf-8'))
    pcd_file.write(f'TYPE {TYPE}\n'.encode('utf-8'))
    pcd_file.write(f'COUNT {COUNT}\n'.encode('utf-8'))
    pcd_file.write(f'WIDTH {str(point_num)}\n'.encode('utf-8'))
    pcd_file.write('HEIGHT 1\n'.encode('utf-8'))
    pcd_file.write('VIEWPOINT 0 0 0 1 0 0 0\n'.encode('utf-8'))
    pcd_file.write(f'POINTS {str(point_num)}\n'.encode('utf-8'))
    pcd_file.write(f'DATA {type}\n'.encode('utf-8'))

    if type == 'ascii':
        for i in range(point_num):
            point = points[i, :]
            string = ' '.join(map(str, point)) + "\n"
            pcd_file.write(string.encode('utf-8'))

    elif type == 'binary':
        points = array_to_structured_array(points, fields, npdtype)
        pcd_file.write(points.tostring('C'))

    elif type == 'binary_compressed':
        import lzf
        points = array_to_structured_array(points, fields, npdtype)
        uncompressed_lst = []
        for fieldname in points.dtype.names:
            column = np.ascontiguousarray(points[fieldname]).tostring('C')
            uncompressed_lst.append(column)
        uncompressed = b''.join(uncompressed_lst)
        uncompressed_size = len(uncompressed)
        # print("uncompressed_size = %r"%(uncompressed_size))
        buf = lzf.compress(uncompressed)
        if buf is None:
            # compression didn't shrink the file
            # TODO what do to do in this case when reading?
            buf = uncompressed
            compressed_size = uncompressed_size
        else:
            compressed_size = len(buf)
        fmt = 'II'
        pcd_file.write(struct.pack(fmt, compressed_size, uncompressed_size))
        pcd_file.write(buf)
    else:
        raise ValueError("Only 'ascii', 'binary', 'binary_compressed' can be selected for type")

    pcd_file.close()

def extract_bev_points_by_mask(points, mask, range, other_index=-1):
    x_range = range[2]-range[0]
    y_range = range[3]-range[1]

    height, width = mask.shape
    assert width /x_range  == height / y_range
    pixel = height / y_range

    map_index = []
    for point in points:
        if (point[0] > range[0]) and (point[0] < range[2]) and (point[1] > range[1]) and (point[1] < range[3]):
            mask_x = round((point[0] - range[0]) * pixel)
            mask_y = round((range[3] - point[1]) * pixel)

            if mask_x < (range[2] - range[0]) * pixel and mask_y < (range[3] - range[1]) * pixel:
                map_index.append(mask[mask_x][mask_y])
            else:
                map_index.append(other_index)
        else:
            map_index.append(other_index)
    return map_index



# from wata.pointcloud.pcd import PointCloudProcess
#
# import os
# from tabulate import tabulate
# from wata import obtain_wata_path
# import sys
# from pathlib import Path


# def wata_list(argv):
#     assert argv.__len__() == 2, 'error command'
#     table_list = [
#         ['终端命令', '功能'],
#         ['wata', 'Hello WATA !'],
#         ['wata show_kitti', '展示kitti数据集第一帧'],
#         ['wata list', '列出wata终端命令'],
#         ['wata install', '用清华源安装包'],
#         ['wata unzip', 'ubuntu解压文件'],
#         ['wata show_size', 'ubuntu查看子目录的大小'],
#         ['wata.lxq.yanhua', '打开烟花'],
#     ]
#     print(tabulate(table_list, headers='firstrow', tablefmt='grid'))


# def show_kitti(argv):
#     assert argv.__len__() == 2, 'error command'
#     cur_path = obtain_wata_path()
#     PointCloudProcess.show_pcd(os.path.join(cur_path, "resources/pcd/000000.bin"))
#
#
# def fireworks():
#     try:
#         from wata.lxq.fireworks_explosion import open_app as open_fireworks
#         cur_path = obtain_wata_path()
#         open_fireworks(ui=os.path.join(cur_path, "resources/fireworks/main.ui"),
#                        icon=os.path.join(cur_path, "resources/fireworks/icon.png"),
#                        snow=os.path.join(cur_path, "resources/fireworks/snow.gif"),
#                        emoji=os.path.join(cur_path, "resources/fireworks/paitou.gif"),
#                        fireworks=os.path.join(cur_path, "resources/fireworks/fireworks"))
#     except:
#         print("package err")


# def pip_install(argv):
#     assert argv.__len__() > 2
#     package = ''
#     len_pack = argv.__len__()
#     for i in range(2, len_pack):
#         package = package + argv[i] + ' '
#     print("pip install " + package + "-i https://pypi.tuna.tsinghua.edu.cn/simple/")
#     os.system("pip install " + package + "-i https://pypi.tuna.tsinghua.edu.cn/simple/")


# def unzip(argv):
#     user = '' if argv[1] == 'unzip' else 'sudo '
#     assert argv.__len__() > 2
#     zip_file = argv[2]
#     zip_ext = Path(zip_file).suffix[1:]
#     print(zip_ext)
#     if zip_ext == "zip":
#         print(user + "unzip " + zip_file)
#         os.system(user + "unzip " + zip_file)
#     elif zip_ext == "tar":
#         print(user + "tar -xvf " + zip_file)
#         os.system(user + "tar -xvf " + zip_file)
#     elif zip_ext == "tgz":
#         print(user + "tar -xzvf " + zip_file)
#         os.system(user + "tar -xzvf " + zip_file)
#     elif zip_file.split(".")[-1] == "gz" and zip_file.split(".")[-2] == "tar":
#         print(user + "tar -zxvf " + zip_file)
#         os.system(user + "tar -zxvf " + zip_file)
#     else:
#         print('Unable to decompress the file type temporarily')


# def wata_console():
#     if sys.argv.__len__() == 1:
#         print("Hello WATA !")
#         print('Enter "wata list" to view the function')
#         return None
#
#     cmd = sys.argv[1]
#     if cmd == 'install':
#         pip_install(sys.argv)
#         return None
#
#     elif cmd == 'unzip' or (cmd == 'sudo' and sys.argv[2] == 'unzip'):
#         unzip(sys.argv)
#         return None
#
#     elif cmd == 'list':
#         wata_list(sys.argv)
#         return None
#
#     elif cmd == 'show_kitti':
#         show_kitti(sys.argv)
#         return None
#
#     elif cmd == 'show_size':
#         os.system("sudo du -h --max-depth=1")
#         return None
#     else:
#         print("error command")



from wata.pointcloud.utils import utils
from wata.pointcloud.utils.o3d_visualize_utils import open3d_draw_scenes, show_pcd_from_points_by_open3d,show_o3d_model
from wata.pointcloud.utils.plot_visualize_utils import plot_draw_scenes, show_pcd_from_points_by_matplotlib
from wata.pointcloud.utils.qtopengl_visualize_utils import show_pcd_from_points_by_qtopengl
from numpy import ndarray
from pathlib import Path
from typing import Union




class PointCloudVisualize:

    @staticmethod
    def show_pcd(path: Union[str, Path], point_size: float = 1, background_color: list = None, pcd_range: list = None,
                 bin_num_features: int = None,
                 create_coordinate: bool = True, create_plane: bool = False, type: str = 'open3d'):
        '''
        **功能描述**: 直接可视化点云文件,目前支持bin pcd npy 格式的点云

        Args:
            path: 点云的路径
            point_size: 点的大小
            background_color: 背景颜色
            pcd_range: 可视化点云的范围
            bin_num_features: 如果是bin格式的文件,应给出bin格式点云的列数
            create_coordinate: 是否可视化坐标系
            create_plane: 是否可视化出一个平面
            type: 用什么工具可视化, 默认open3d 可选 matplotlib open3d qtopengl mayavi(未开发) vispy(未开发)

        Returns:
            直接弹出可视化界面
        '''
        utils.show_pcd(path, point_size, background_color, pcd_range, bin_num_features,
                       create_coordinate, create_plane, type)

    @staticmethod
    def show_pcd_from_points(points: Union[ndarray, list], point_size: float = 1, background_color: list = None,
                             colors: Union[list, int] = None, create_coordinate=True,
                             create_plane=False, type='open3d'):
        '''
        **功能描述**: 展示点云

        Args:
            points: numpy格式的点云;当points为list时代表将list中的numpy格式的点云全部可视化
            point_size: 可视化点的大小,默认为1
            background_color: 背景颜色
            colors: 点云的颜色,当colors为int类型时,为用点云的第几列显色;当colors为list类型时,colors代表[R,G,B]
            create_coordinate: 是否可视化坐标系 , 可以设置数字调整坐标轴的尺寸,默认大小为1
            create_plane: 是否可视化出一个平面 ,默认每个网格宽度为10m, 可以设置数调整每个网格的尺寸
            type: 用什么工具可视化, 默认open3d 可选 matplotlib open3d qtopengl mayavi(未开发) vispy(未开发)

        Returns:
            无
        '''
        utils.show_pcd_from_points(points, point_size, background_color, colors, create_coordinate, create_plane, type)

    @staticmethod
    def show_pcd_from_points_by_open3d(points: Union[ndarray, list], point_size: float = 1, background_color: list = [0,0,0],
                             colors: Union[list, int] = None, create_coordinate=True,create_plane=False,cam_param=None, window_size=[1200,800]):
        show_pcd_from_points_by_open3d(points, point_size, background_color, colors, create_coordinate,
                                   create_plane, cam_param, window_size)

    @staticmethod
    def show_pcd_from_points_by_matplotlib(points: Union[ndarray, list], point_size: float = 1, background_color: list = None,
                             colors: Union[list, int] = None, create_coordinate=True,savepath=None, plot_range=None):
        show_pcd_from_points_by_matplotlib(points, point_size, background_color, colors,
                                       create_coordinate, savepath, plot_range)
    @staticmethod
    def show_pcd_from_points_by_qtopengl(points, point_size=1.5, background_color=[0,0,0], create_coordinate=True, create_plane=True):
        show_pcd_from_points_by_qtopengl(points, point_size, background_color, create_coordinate, create_plane)

    @staticmethod
    def add_boxes(points: Union[ndarray, list], gt_boxes: Union[ndarray, None] = None,
                  gt_labels: Union[None, list] = None, pred_boxes: Union[ndarray, None] = None,
                  pred_labels: Union[None, list] = None, pred_scores: Union[None, list] = None,
                  point_size: int = 1,
                  background_color: list = None, create_plane: bool = False, point_colors: Union[None, list] = None,
                  create_coordinate=True, type='open3d',
                  savepath=None, plot_range=None):
        '''
        **功能描述**: 绘制点云、标注框、预测框

        Args:
            points: numpy格式的点云;当points为list时代表将list中的numpy格式的点云全部可视化
            gt_boxes: GT框
            gt_labels: GT框标签
            pred_boxes: 预测框
            pred_labels: 预测框标签
            pred_scores: 预测框的分数
            point_size: 可视化点的大小,默认为1
            background_color: 背景颜色
            create_plane: 是否创建一个可视化地面
            point_colors: 对应points为列表时的颜色,当points为numpy格式时不起作用
            create_coordinate: 是否在原点创建坐标系
            type: 用什么工具可视化, 默认open3d 可选 matplotlib open3d qtopengl mayavi(未开发) vispy(未开发)
            savepath: 当type为matplotlib时,保存图像的地址
            plot_range: 当type为matplotlib时,可视化的范围

        Returns:
            可视化界面
        '''
        utils.add_boxes(points, gt_boxes=gt_boxes, gt_labels=gt_labels, pred_boxes=pred_boxes, pred_labels=pred_labels,
                        pred_scores=pred_scores, point_size=point_size,
                        background_color=background_color, create_plane=create_plane, point_colors=point_colors,
                        create_coordinate=create_coordinate, type=type, savepath=savepath, plot_range=plot_range)

    @staticmethod
    def add_boxes_by_open3d(points, colors=None, gt_boxes=None, gt_labels=None, pred_boxes=None, pred_labels=None,
                       pred_scores=None,point_size=1, background_color=None, create_plane=True,
                       create_coordinate=True, cam_param=None, window_size=[1200, 800]):
        open3d_draw_scenes(points, colors, gt_boxes, gt_labels, pred_boxes, pred_labels,
                       pred_scores,point_size, background_color, create_plane,
                       create_coordinate, cam_param, window_size)

    @staticmethod
    def add_boxes_by_matplotlib(points, gt_boxes=None, gt_labels=None, pred_boxes=None, pred_labels=None,
                                pred_scores=None, point_size=1,
                                background_color=None, colors=None, create_coordinate=True, savepath=None, plot_range=None):
        plot_draw_scenes(points=points, gt_boxes=gt_boxes, gt_labels=gt_labels,
                         pred_boxes=pred_boxes, pred_labels=pred_labels, pred_scores=pred_scores,
                         point_size=point_size, background_color=background_color,
                         point_colors=colors, create_coordinate=create_coordinate, savepath=savepath, plot_range=plot_range)
    @staticmethod
    def show_o3d_model(o3d_model_list, point_size, background_color=[0,0,0], cam_param=None, window_size=[1200, 800]):
        show_o3d_model(o3d_model_list, point_size, background_color, cam_param, window_size)


try:
    import cv2
except:
    OPENCV = None
import os
from PIL import Image
import numpy as np
from pathlib import Path

def show_img(path):
    img = cv2.imread(path)
    cv2.imshow('show_img', img)
    cv2.waitKey(0)
    cv2.destroyAllWindows()

def images_to_video(img_dir, save_path, fps=30):
    supported_formats = (".tif", ".png", ".jpg", ".jpeg")
    # Retrieve the paths of all eligible files in the image folder and sort them by file name
    images = [img for img in os.listdir(img_dir) if img.lower().endswith(supported_formats)]
    images.sort(key=lambda x: x)

    first_image_path = os.path.join(img_dir, images[0])
    first_image = Image.open(first_image_path)
    img_size = first_image.size
    fourcc = cv2.VideoWriter_fourcc(*'mp4v')
    out = cv2.VideoWriter(save_path, fourcc, fps, img_size)

    for image in images:
        image_path = os.path.join(img_dir, image)
        img = Image.open(image_path)
        img_array = np.array(img)
        assert img_array.shape[0] == img_size[0], "Inconsistent image size."
        assert img_array.shape[1] == img_size[1], "Inconsistent image size."
        if img_array.shape[2] == 4:
            img_array = img_array[:, :, :3]
        out.write(img_array)
    out.release()
    print(save_path, "已保存")


def video_to_images(path, save_path, img_format='jpg'):

    file_name = Path(path).stem

    if not os.path.exists(save_path):
        os.makedirs(save_path)

    cap = cv2.VideoCapture(path)

    frame_count = 0
    while True:
        ret, frame = cap.read()
        if not ret:
            break

        output_image_path = os.path.join(save_path, f"{file_name}_{frame_count:04d}.{img_format}")

        cv2.imwrite(output_image_path, frame)

        frame_count += 1

    cap.release()
    print(f"视频已成功分解为{frame_count}张图片，保存于{save_path}。")




from wata.geo.utils import utils, coord_trans

class GeoProcess:
    
    @staticmethod
    def WGS84_to_GCJ02(lat, lon):
        '''
        **功能描述**: WGS84_to_GCJ02  
        
        Args:
            lat: WGS84坐标系的纬度  
            lon: WGS84坐标系的经度  

        Returns:
            GCJ02坐标系的纬度,GCJ02坐标系的经度  
        '''
        return coord_trans.WGS84_to_GCJ02(lat, lon)
    @staticmethod
    def WGS84_to_BD09(lat, lon):
        '''
        **功能描述**: WGS84_to_BD09  
        
        Args:
            lat: WGS84坐标系的纬度  
            lon: WGS84坐标系的经度  

        Returns:
            BD09坐标系的纬度,BD09坐标系的经度  
        '''
        return coord_trans.WGS84_to_BD09(lat, lon)
    @staticmethod
    def WGS84_to_UTM(lat:float, lon:float):
        '''
        **功能描述**: WGS84_to_UTM  
        
        Args:
            lat: WGS84坐标系的纬度  
            lon: WGS84坐标系的经度  

        Returns:
            UTM坐标  
        '''
        return coord_trans.WGS84_to_UTM(lat, lon)

    @staticmethod
    def GCJ02_to_WGS84(lat, lon):
        '''
        **功能描述**: GCJ02_to_WGS84  
        
        Args:
            lat: GCJ02坐标系的纬度  
            lon: GCJ02坐标系的经度  

        Returns:  
            WGS84坐标系的纬度,WGS84坐标系的经度  
        '''
        return coord_trans.GCJ02_to_WGS84(lat, lon)
    @staticmethod
    def GCJ02_to_BD09(lat, lon):
        '''
        **功能描述**: GCJ02_to_BD09  
        
        Args:  
            lat: GCJ02坐标系的纬度  
            lon: GCJ02坐标系的经度  

        Returns:  
            BD09坐标系的纬度,BD09坐标系的经度  
        '''
        return coord_trans.GCJ02_to_BD09(lat, lon)

    @staticmethod
    def BD09_to_GCJ02(lat, lon):
        '''
        **功能描述**: BD09_to_GCJ02  
        
        Args:
            lat: BD09坐标系的纬度  
            lon: BD09坐标系的经度  

        Returns:
            GCJ02坐标系的纬度,GCJ02坐标系的经度  
        '''
        return coord_trans.BD09_to_GCJ02(lat, lon)
    @staticmethod
    def BD09_to_WGS84(lat, lon):
        '''
        **功能描述**: BD09_to_WGS84  
        
        Args:
            lat: BD09坐标系的纬度  
            lon: BD09坐标系的经度  

        Returns:
            WGS84坐标系的纬度,WGS84坐标系的经度  
        '''
        return coord_trans.BD09_to_WGS84(lat, lon)
    
    @staticmethod
    def UTM_to_WGS84(easting, northing, zone_number, zone_letter):
        '''
        **功能描述**: UTM_to_WGS84  
        
        Args:  
            easting:  
            northing:  
            zone_number: 区域ID  
            zone_letter:  

        Returns:
            WGS84坐标系的纬度,WGS84坐标系的经度  
        '''
        return coord_trans.UTM_to_WGS84(easting, northing, zone_number, zone_letter)


from wata.file.utils import utils, file_tree_utils # 不能去掉
from pathlib import Path
from numpy import ndarray
from typing import Union

class FileProcess:
    @staticmethod
    def load_file(path:Union[str, Path]):
        '''
        **功能描述**: 读取文件,支持 txt pkl json geojson yaml 格式  
        
        Args:
            path: 加载文件的路径  

        Returns:
            文件内容  
        '''
        return utils.load_file(path)

    @staticmethod
    def save_file(data, save_path):
        '''
        **功能描述**: 保存文件 支持'yaml', 'json', 'pkl', 'txt' 格式  
        
        Args:
            data: 文件内容  
            save_path: 文件保存路径  

        Returns:  
            无  

        '''
        utils.save_file(data, save_path)

    @staticmethod
    def write_file(data, save_path):
        '''
        **功能描述**: 保存文件 支持'yaml', 'json', 'pkl', 'txt' 格式  
        
        Args:
            data: 文件内容  
            save_path: 文件保存路径  

        Returns:
            无

        '''
        utils.save_file(data, save_path)

    @staticmethod
    def file_tree(path, save_txt=None):
        '''
        **功能描述**: 列出文件树  
        
        Args:
            path: 需要列出文件树的路径  
            save_txt: 保存txt时填写为地址,不保存时为None,默认为None  

        Returns:
            无

        '''
        file_tree_utils.file_tree(path, save_txt)

    @staticmethod
    def mkdir(dir_path):
        '''
        **功能描述**: 如果不存在这个目录,则创建  
        
        Args:
            dir_path: 目录地址  

        Returns:
            无
        '''
        utils.mkdir_if_not_exist(dir_path)
    @staticmethod
    def mkdir_if_not_exist(dir_path):
        '''
        **功能描述**: 如果不存在这个目录,则创建  
        
        Args:
            dir_path: 目录地址  

        Returns:
            无  
        '''
        utils.mkdir_if_not_exist(dir_path)

    @staticmethod
    def np2str(arr:Union[ndarray,list], return_type:Union[str, list]):
        '''
        **功能描述**: 将numpy 或 list 格式的矩阵转成字符串或者用list存储的字符串  
        
        Args:
            arr: numpy 或者 list 格式的矩阵  
            return_type: 可选"str","list"  

        Returns:  
            list | str  
        '''
        return utils.np2str(arr,return_type)

    @staticmethod
    def is_empty_directory(directory):
        '''
        **功能描述**: 判断路径是否为空目录

        Args:
            directory: 指定的路径

        Returns:
            True | False
        '''
        return utils.is_empty_directory(directory)

    @staticmethod
    def delete_empty_directories(directory):
        '''
        **功能描述**: 删除指定路径下的空目录

        Args:
            directory: 指定的路径

        Returns:
            None
        '''
        return utils.delete_empty_directories(directory)

    @staticmethod
    def get_sorted_files(directory="."):
        '''
        **功能描述**: 获取指定路径下的文件，并排序

        Args:
            directory: 指定的路径

        Returns:
            排序的文件名列表
        '''
        return utils.get_sorted_files(directory)

    @staticmethod
    def get_sorted_directories(directory="."):
        '''
        **功能描述**: 获取指定路径下的文件，并排序

        Args:
            directory: 指定的路径

        Returns:
            排序的文件名列表
        '''
        return utils.get_sorted_directories(directory)



try:
    from PyQt5.QtWidgets import QApplication, QPushButton, QVBoxLayout, QWidget, QLabel
    from PyQt5.QtOpenGL import QGLWidget
    from PyQt5.QtCore import Qt
    from PyQt5 import QtCore, QtGui
    from PyQt5.QtGui import QColor
    import pyqtgraph.opengl as gl
    from pyqtgraph.opengl import GLAxisItem
except:
    PYQT5 = None

import numpy as np
from scipy.spatial.distance import cdist
import matplotlib.pyplot as plt

def quit_application():
    QApplication.quit()


def show_pcd_from_points_by_qtopengl(points, point_size, background_color, create_coordinate=True, create_plane=True):
    points = points[:, 0:3]
    app = QApplication([])

    """创建主窗口部件"""
    window = QWidget()
    window.resize(1200, 800)

    """创建OpenGL视图窗口"""
    widget = gl.GLViewWidget()
    widget.setWindowTitle('Point Cloud Viewer')

    Colors = plt.get_cmap('cool')  # 参考:https://zhuanlan.zhihu.com/p/114420786
    colors = Colors(points[:, 0] / 40)

    """添加点云"""
    scatter = gl.GLScatterPlotItem(pos=points, color=colors, size=point_size)
    widget.addItem(scatter)

    """添加平面网格"""
    if create_plane:
        if create_plane == True:
            create_plane = 10
        plane = gl.GLGridItem(size=QtGui.QVector3D(200, 200, 0), color=QColor(40, 40, 40, 255), glOptions='translucent')
        plane.setSpacing(create_plane, create_plane, 1)
        plane_center = (0, 0, -3)
        plane.translate(*plane_center)
        widget.addItem(plane)

    '''添加坐标系'''
    if create_coordinate:
        if create_coordinate == True:
            create_coordinate = 3
        axis = gl.GLAxisItem(size=QtGui.QVector3D(create_coordinate, create_coordinate, create_coordinate))
        widget.addItem(axis)

    """调整视角"""
    widget.opts['distance'] = 80
    widget.setCameraPosition(distance=widget.opts['distance'], elevation=15, azimuth=-110)

    """设置背景颜色"""
    widget.setBackgroundColor(background_color[0], background_color[1], background_color[2], 1)

    layout = QVBoxLayout()
    layout.addWidget(widget)

    window.setLayout(layout)
    window.show()
    # 运行应用程序
    app.exec_()

# # 从文件中读取点云数据
# point_cloud = o3d.io.read_point_cloud("../example.pcd")
# points = np.asarray(point_cloud.points)
# Colors = plt.get_cmap('cool')  # 参考:https://zhuanlan.zhihu.com/p/114420786
# colors = Colors(points[:, 0] / 40)

# """创建PyQt应用程序对象"""
# app = QApplication([])

# """创建主窗口部件"""
# window = QWidget()
# window.resize(800, 600)

# """创建OpenGL视图窗口"""
# widget = gl.GLViewWidget()
# widget.setWindowTitle('Point Cloud Viewer')

# """添加网格项到视图窗口中"""
# grid = gl.GLGridItem()
# widget.addItem(grid)

# """调整视角"""
# widget.opts['distance'] = 40
# widget.setCameraPosition(distance=widget.opts['distance'], elevation=15, azimuth=45)

# """添加点云到视图窗口中"""
# point_size = 2  # 设置点的大小
# scatter = gl.GLScatterPlotItem(pos=points, color=colors, size=point_size)
# widget.addItem(scatter)

# """画线的方法"""
# box_center = np.array([0, 0, 0])  # 框的中心点
# box_size = np.array([1, 1, 1])  # 框的尺寸
# box_pos = box_center - box_size / 2
# box_lines = np.array([  # 就是把下面的点按照顺序连接起来
#     [box_pos[0], box_pos[1] + 0.5, box_pos[2] - 0.5], [box_pos[0], box_pos[1] - 0.5, box_pos[2] + 0.5],
#     [box_pos[0], box_pos[1] + 0.5, box_pos[2] + 0.5]  # lower edges
# ])
# box_color = (1, 0, 0, 1)  # 设置线的颜色
# box_width = 2  # 设置线的宽度
# box_lines_item = gl.GLLinePlotItem(pos=box_lines, color=box_color, width=box_width)
# widget.addItem(box_lines_item)

# """画3d线框的方法"""
# '''
# 感觉没啥用啊！
# 'additive'：将盒子以加法混合模式渲染，形成颜色累积效应。
# 'opaque'：将盒子设置为不透明。
# 'translucent'：将盒子设置为半透明。
# '''
# box = gl.GLBoxItem(size=QtGui.QVector3D(1, 2, 1), color=QColor(255, 255, 0, 255), glOptions='opaque')
# widget.addItem(box)
# # 设置立方体的中心点位置
# box_center = (1, 1, 0)
# box.translate(*box_center)
# """ 
# Rotate the object around the axis specified by (x,y,z).
# *angle* is in degrees.
# """
# box.rotate(90, 0, 0, 1)


# """添加平面网格"""
# plane = gl.GLGridItem(size=QtGui.QVector3D(10, 20, 1), color=QColor(255, 0, 0, 255), glOptions='translucent')
# plane_center = (1, 10, 1)
# plane.translate(*plane_center)
# widget.addItem(plane)


# ''' 创建mesh平面, 可以用它来画3d实心框'''
# vertexes = [[0, 0, 0], [0, 2, 2], [0, 0, 2], [0, 2, 0]]
# faces = [[0, 1, 2], [1, 0, 3]]
# meshface = gl.MeshData(vertexes=np.array(vertexes), faces=np.array(faces))
# meshface = gl.GLMeshItem(meshdata=meshface, smooth=False, shader='shaded', glOptions='translucent', drawEdges=True,
#                          color=QColor(255, 0, 255, 200))
# # meshface.translate(1, 0, 0)
# widget.addItem(meshface)

# text_label = gl.GLTextItem(text="3D 框", pos=(0, 0, 0), color=QColor(QtCore.Qt.blue),
#                            font=QtGui.QFont('Helvetica', 16))

# widget.addItem(text_label)


# '''创建退出按钮'''
# quit_button = QPushButton("退出")
# quit_button.clicked.connect(quit_application)

# # 创建垂直布局，并将OpenGL视图窗口和退出按钮添加到其中
# layout = QVBoxLayout()
# layout.addWidget(widget)
# layout.addWidget(quit_button)

# # 将布局设置给主窗口部件
# window.setLayout(layout)
# window.show()

# # 运行应用程序
# app.exec_()



from wata.mathematics.utils import utils
from wata.mathematics.utils import trans_coordinate
import numpy as np

class Maths:

    @staticmethod
    def point_in_polygon(point:list, polygon:list):
        '''
        **功能描述**:判断点是否在多边形内  
        
        Args:
            point: 平面中的点  
            polygon: 含多边形中的点的列表  

        Returns:
            返回 1 在多边形内,返回0不在多边形内  

        '''
        return utils.point_in_polygon(point, polygon)

    @staticmethod
    def xy2rt(x:float,y:float):
        '''
        **功能描述**: 平面直角坐标系转极坐标系  
        
        Args:
            x: 平面直角坐标系中的横坐标值  
            y: 平面直角坐标系中的纵坐标值  

        Returns:  
            极坐标系中的坐标 (r,theta)  
        '''
        return trans_coordinate.xy2rt(x,y)

    @staticmethod
    def rt2xy(r, theta):
        '''
        **功能描述**: 极坐标系转平面直角坐标系  
        
        Args:  
            r: 极坐标系中的原点距离  
            theta: 极坐标系中的极角  

        Returns:  
            平面直角坐标系(x,y)  

        '''
        return trans_coordinate.rt2xy(r, theta)

    @staticmethod
    def xyz2rtp(x, y, z):
        '''
        **功能描述**: 笛卡尔直角坐标系转球坐标系  
        
        Args:  
            x: 笛卡尔直角坐标系中的x值  
            y: 笛卡尔直角坐标系中的y值  
            z: 笛卡尔直角坐标系中的z值  

        Returns:
            球坐标系 r, theta, phi  
        '''
        return trans_coordinate.xyz2rtp(x, y, z)

    @staticmethod
    def rtp2xyz(r,theta,phi):
        '''
        **功能描述**: 球坐标系转笛卡尔坐标系  
        
        Args:  
            r: 原点距离  
            theta: 水平偏角  
            phi: 竖直偏角  

        Returns:  
            笛卡尔坐标系(x,y,z)  

        '''
        return trans_coordinate.rtp2xyz(r,theta,phi)


#!/usr/bin/env python3
import warnings
import struct
import numpy as np
import re
import lzf

numpy_pcd_type_mappings = [(np.dtype('float32'), ('F', 4)),
                           (np.dtype('float64'), ('F', 8)),
                           (np.dtype('uint8'), ('U', 1)),
                           (np.dtype('uint16'), ('U', 2)),
                           (np.dtype('uint32'), ('U', 4)),
                           (np.dtype('uint64'), ('U', 8)),
                           (np.dtype('int16'), ('I', 2)),
                           (np.dtype('int32'), ('I', 4)),
                           (np.dtype('int64'), ('I', 8))]
numpy_type_to_pcd_type = dict(numpy_pcd_type_mappings)
pcd_type_to_numpy_type = dict((q, p) for (p, q) in numpy_pcd_type_mappings)


def parse_ascii_pc_data(f, dtype, metadata):
    """ Use numpy to parse ascii pointcloud data.
    """
    return np.loadtxt(f, dtype=dtype, delimiter=' ')


def parse_binary_pc_data(f, dtype, metadata):
    rowstep = metadata['points'] * dtype.itemsize
    # for some reason pcl adds empty space at the end of files
    buf = f.read(rowstep)
    return np.frombuffer(buf, dtype=dtype)


def parse_binary_compressed_pc_data(f, dtype, metadata):
    """ Parse lzf-compressed data.
    Format is undocumented but seems to be:
    - compressed size of data (uint32)
    - uncompressed size of data (uint32)
    - compressed data
    - junk
    """
    fmt = 'II'
    compressed_size, uncompressed_size = \
        struct.unpack(fmt, f.read(struct.calcsize(fmt)))
    compressed_data = f.read(compressed_size)
    # TODO what to use as second argument? if buf is None
    # (compressed > uncompressed)
    # should we read buf as raw binary?
    buf = lzf.decompress(compressed_data, uncompressed_size)
    if len(buf) != uncompressed_size:
        raise IOError('Error decompressing data')
    # the data is stored field-by-field
    pc_data = np.zeros(metadata['width'], dtype=dtype)
    ix = 0
    for dti in range(len(dtype)):
        dt = dtype[dti]
        bytes = dt.itemsize * metadata['width']
        column = np.fromstring(buf[ix:(ix + bytes)], dt)
        pc_data[dtype.names[dti]] = column
        ix += bytes
    return pc_data


def parse_header(lines):
    """ Parse header of PCD files.
    """
    metadata = {}
    for ln in lines:
        if ln.startswith('#') or len(ln) < 2:
            continue
        match = re.match('(\w+)\s+([\w\s\.]+)', ln)
        if not match:
            warnings.warn("warning: can't understand line: %s" % ln)
            continue
        key, value = match.group(1).lower(), match.group(2)
        if key == 'version':
            metadata[key] = value
        elif key in ('fields', 'type'):
            metadata[key] = value.split()
        elif key in ('size', 'count'):
            metadata[key] = map(int, value.split())
        elif key in ('width', 'height', 'points'):
            metadata[key] = int(value)
        elif key == 'viewpoint':
            metadata[key] = map(float, value.split())
        elif key == 'data':
            metadata[key] = value.strip().lower()
        # TODO apparently count is not required?
    # add some reasonable defaults
    if 'count' not in metadata:
        metadata['count'] = [1] * len(metadata['fields'])
    if 'viewpoint' not in metadata:
        metadata['viewpoint'] = [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0]
    if 'version' not in metadata:
        metadata['version'] = '.7'
    return metadata


def _build_dtype(metadata):
    """ Build numpy structured array dtype from pcl metadata.
    Note that fields with count > 1 are 'flattened' by creating multiple
    single-count fields.
    *TODO* allow 'proper' multi-count fields.
    """
    fieldnames = []
    typenames = []
    idx = 0
    for f, c, t, s in zip(metadata['fields'],
                          metadata['count'],
                          metadata['type'],
                          metadata['size']):
        # if f=='_':
        #     continue
        idx += 1
        np_type = pcd_type_to_numpy_type[(t, s)]
        if c == 1:
            fieldnames.append(f)
            typenames.append(np_type)
        else:
            if f == '_':
                fieldnames.extend(['%s%s_%04d_%d' % (f, c, i, idx) for i in range(c)])
                typenames.extend([np_type] * c)
            else:
                fieldnames.extend(['%s%s_%04d' % (f, c, i) for i in range(c)])
                typenames.extend([np_type] * c)
    dtype = np.dtype(list(zip(fieldnames, typenames)))
    return dtype


def points_from_fileobj(f):
    """ Parse pointcloud coming from file object f
    """
    header = []
    while True:
        ln = f.readline().strip()
        ln_str = ln.decode(encoding='utf-8', errors='strict')
        header.append(ln_str)
        if ln_str.startswith('DATA'):
            metadata = parse_header(header)
            dtype = _build_dtype(metadata)
            break
    if metadata['data'] == 'ascii':
        pc_data = parse_ascii_pc_data(f, dtype, metadata)
    elif metadata['data'] == 'binary':
        pc_data = parse_binary_pc_data(f, dtype, metadata)
    elif metadata['data'] == 'binary_compressed':
        pc_data = parse_binary_compressed_pc_data(f, dtype, metadata)
    else:
        raise ValueError('DATA field is neither "ascii" or "binary" or\
                "binary_compressed"')
    return pc_data, metadata

def get_metadata_from_pcd_file(file_path):
    with open(file_path, 'rb') as f:
        header = []
        while True:
            ln = f.readline().strip()
            ln_str = ln.decode(encoding='utf-8', errors='strict')
            header.append(ln_str)
            if ln_str.startswith('DATA'):
                metadata = parse_header(header)
                break
    return metadata

def get_points_from_pcd_file(file_path, num_features=None):
    points, metadata = get_points(file_path)

    fields = [item for item in metadata['fields'] if item != "_"]
    if num_features is None:
        num_features = len(fields)
    points_ = points[fields[0]].reshape(-1, 1)

    i = 0
    for field in fields[1:num_features]:
        points_ = np.hstack((points_, points[field].reshape(-1, 1)))
        i +=1

    return points_


def get_points(file_path):
    with open(file_path, 'rb') as f:
        points, metadata = points_from_fileobj(f)
    return points, metadata



# from decodelidardata import decode_lidar

import pathlib
import pandas as pd
import numpy as np
from numpy.lib import recfunctions
import wata

LIDAR_COLUMN_NAMES = ['x', 'y', 'z', 'intensity', 'lidar_identifier', 'delta_time',
                      'Class', 'ID', 'lidar_mat_id', 'normal_cos', 'part_id', 'point_idx',
                      'reference_reflectivity', 'nominator', 'denominator', 'reference_distance',
                      'normal angle theta', 'normal angle phi']

MINUS_FILL = ['Class', 'ID']

ZERO_FILL = ['x', 'y', 'z', 'intensity', 'lidar_identifier', 'delta_time', 'lidar_mat_id', 'normal_cos',
             'part_id', 'point_idx', 'reference_reflectivity', 'nominator', 'denominator', 'reference_distance',
             'normal angle theta', 'normal angle phi']

DROPPED_FIELDS = ['classInstance', 'StructurePointindex', 'fillfactor', 'place holder1',
                  'place holder2', 'place holder3', 'place holder4', 'place holder5', 'place holder6',
                  'place holder7']


def decodeLidarFile(file, withMeta=True):
    """
    Previous signature, kept for backwards compatibility

    :param file: full path to point_cloud file
    :param withMeta: depricated bool
    :return:
    """
    return decode_lidar(file)


def get_lidar_dtype(extension: str) -> np.dtype:
    """
    Constructs a point_cloud parsing np.dtype according to file extension

    :param extension: point_cloud file extension, valid exentions are 'ldo'/'ldg'/'ldx',
    passing an invalid string will raise a ValueError

    :return: np.dtype that can be used to parse the file with all fields
    """
    if extension not in ["ldo", "ldg", "ldx"]:
        raise ValueError(f"Invalid flie type passed: {extension}")

    lidar_dtype = [('x', np.float32), ('y', np.float32), ('z', np.float32),
                   ('delta_time', np.float16),
                   ('lidar_identifier', np.uint8),
                   ('intensity', np.uint8)]

    if extension != 'ldo':
        lidar_dtype.extend([('classInstance', np.uint32),
                            ('lidar_mat_id', np.uint16),
                            ('normal_cos', np.float16),
                            ('StructurePointindex', np.uint32),
                            ('reference_distance', np.float16),
                            ('fillfactor', np.uint8),
                            ('reference_reflectivity', np.uint8)])
        if extension != 'ldg':
            lidar_dtype.extend([('normal angle theta', np.float16),
                                ('normal angle phi', np.float16),
                                ('place holder1', np.float32),
                                ('place holder2', np.float32),
                                ('place holder3', np.float32),
                                ('place holder4', np.float32),
                                ('place holder5', np.float32),
                                ('place holder6', np.float32),
                                ('place holder7', np.float32)])

    return np.dtype(lidar_dtype)


def decode_lidar(file: str, extension: str = '') -> pd.DataFrame:
    """
    Decode Lidar binary 拼接点云.py and returns pd dataframe

    :param file: absolute path to binary point_cloud 拼接点云.py file
    :param extension: optional - indicates type of point_cloud file (valid values: 'ldo','ldg','ldx')

    :return: dataframe holding the parsed point_cloud 拼接点云.py
    """
    if not extension:
        extension = pathlib.Path(file).suffix[1:]
    lidar_dtype = get_lidar_dtype(extension)
    data = np.fromfile(file, lidar_dtype)
    if extension != 'ldo':
        # separate combo values
        object_class = data['classInstance'] >> 24
        object_instance = data['classInstance'] & 0xffffff
        object_structure = data['StructurePointindex'] >> 24
        object_pointindex = data['StructurePointindex'] & 0xffffff
        fill_nominator = (data['fillfactor'] >> 4) + 1
        fill_denominator = (data['fillfactor'] & 0xf) + 1
        # remove unneeded fields and add intensity and ring to 拼接点云.py structure
        data = recfunctions.drop_fields(data, DROPPED_FIELDS)
        data = recfunctions.append_fields(data, ['Class', 'ID', 'part_id', 'point_idx', 'nominator', 'denominator'],
                                          [object_class, object_instance, object_structure, object_pointindex,
                                           fill_nominator, fill_denominator])
    data = pd.DataFrame.from_records(data).reindex(LIDAR_COLUMN_NAMES, axis=1)
    data[ZERO_FILL] = data[ZERO_FILL].fillna(0)
    data[MINUS_FILL] = data[MINUS_FILL].fillna(-1)
    return data.values


import numpy as np
import struct

numpy_pcd_type_mappings = [(np.dtype('float32'), ('F', 4)),
                           (np.dtype('float64'), ('F', 8)),
                           (np.dtype('uint8'), ('U', 1)),
                           (np.dtype('uint16'), ('U', 2)),
                           (np.dtype('uint32'), ('U', 4)),
                           (np.dtype('uint64'), ('U', 8)),
                           (np.dtype('int16'), ('I', 2)),
                           (np.dtype('int32'), ('I', 4)),
                           (np.dtype('int64'), ('I', 8))]

numpy_struct_type_mappings = [(np.dtype('float32'), 'f'),
                              (np.dtype('float64'), 'd'),
                              (np.dtype('uint8'), 'B'),
                              (np.dtype('uint16'), 'H'),
                              (np.dtype('uint32'), 'I'),
                              (np.dtype('uint64'), 'Q'),
                              (np.dtype('int16'), 'h'),
                              (np.dtype('int32'), 'i'),
                              (np.dtype('int64'), 'q')]

np_numpy_type_mappings = [('f32', np.dtype('float32')),
                          ('f64', np.dtype('float64')),
                          ('u8', np.dtype('uint8')),
                          ('u16', np.dtype('uint16')),
                          ('u32', np.dtype('uint32')),
                          ('u64', np.dtype('uint64')),
                          ('i16', np.dtype('int16')),
                          ('i32', np.dtype('int32')),
                          ('i64', np.dtype('int64'))]


def numpy_type_to_pcd_type(key):
    return dict(numpy_pcd_type_mappings)[key]


def pcd_type_to_numpy_type(key):
    return dict((q, p) for (p, q) in numpy_pcd_type_mappings)[key]


def numpy_type_to_struct_type(key):
    return dict(numpy_struct_type_mappings)[key]


def struct_type_to_numpy_type(key):
    return dict((q, p) for (p, q) in numpy_struct_type_mappings)[key]


def np_type_to_numpy_type(key):
    return dict(np_numpy_type_mappings)[key]


def numpy_type_to_np_type(key):
    return dict((q, p) for (p, q) in np_numpy_type_mappings)[key]



from wata.image.img import ImageProcess
from wata.pointcloud.pcd import PointCloudProcess
from wata.file.file import FileProcess
from wata.mathematics.mathematics import Maths
from wata.display.display import Display
from wata.geo.geo import GeoProcess
import os

def __version__():
    return "0.1.2.19"

def obtain_wata_path():
    return os.path.dirname(os.path.abspath(__file__))

def obtain_cur_path_cmd():
    return "os.path.dirname(os.path.abspath(__file__))"



import numpy as np
from scipy.spatial.transform import Rotation


def xyzrpy2RTmatrix(xyz_rpy,seq="xyz", degrees=False):
    assert len(xyz_rpy) == 6
    dx, dy, dz, roll, pitch, yaw = xyz_rpy
    r = Rotation.from_euler(seq, [roll, pitch, yaw], degrees=degrees)
    rotation_matrix = r.as_matrix()
    translation = np.array([dx, dy, dz])
    matrix = np.eye(4)
    matrix[:3, :3] = rotation_matrix
    matrix[:3, 3] = translation
    return matrix

def RTmatrix2xyzrpy(RTmatrix,seq="xyz", degrees=False):
    translation = RTmatrix[:3, 3]
    rotation_matrix = RTmatrix[:3, :3]
    r = Rotation.from_matrix(rotation_matrix)
    rpy = r.as_euler(seq, degrees=degrees)  
    dx, dy, dz = translation
    roll, pitch, yaw = rpy
    return np.array([dx, dy, dz, roll, pitch, yaw])

def move_pcd_with_RTmatrix(points, RTmatrix,inv=False):
    if inv:
        RTmatrix = np.linalg.inv(RTmatrix)
    pcd_trans = points.copy()
    pcd_hm = np.pad(points[:, :3], ((0, 0), (0, 1)), 'constant', constant_values=1)  # (N, 4)
    pcd_hm_trans = np.dot(RTmatrix, pcd_hm.T).T
    pcd_trans[:, :3] = pcd_hm_trans[:, :3]
    return pcd_trans

def rotate_pointcloud(points,rpy,seq="xyz",degrees=True):
    r = Rotation.from_euler(seq, rpy, degrees=degrees)
    rotation_matrix = r.as_matrix()
    matrix = np.eye(4)
    matrix[:3, :3] = rotation_matrix
    new_points = move_pcd_with_RTmatrix(points, matrix)
    return new_points

def translate_pointcloud(points,xyz):
    matrix = np.eye(4)
    matrix[:3, 3] = np.array(xyz)
    new_points = move_pcd_with_RTmatrix(points, matrix)
    return new_points


def move_pcd_with_xyzrpy(points, xyz_rpy,seq, degrees=False):
    assert len(xyz_rpy) == 6
    RT_matrix = xyzrpy2RTmatrix(xyz_rpy,seq, degrees=degrees)
    new_pcd = move_pcd_with_RTmatrix(points, RT_matrix)
    return new_pcd





import os
from pathlib import Path
import numpy as np
from wata.file.utils import load_and_write_file


def load_file(path):
    file_ext = Path(path).suffix[1:]
    if file_ext in ['yaml', 'json', 'geojson', 'pkl', 'txt']:
        return eval('load_and_write_file.load_' + file_ext)(path)
    else:
        raise NameError("Unable to handle {} formatted files".format(file_ext))

def save_file(data, save_path):
    file_ext = Path(save_path).suffix[1:]
    if file_ext in ['yaml', 'json', 'pkl', 'txt']:
        return eval('load_and_write_file.write_' + file_ext)(data, save_path)
    else:
        raise NameError("Unable to handle {} formatted files".format(type))

def mkdir_if_not_exist(path):
    if not os.path.exists(path):
        os.makedirs(path)

def np2str(arr, return_type='list'):
    if isinstance(arr, list):
        arr = np.array(arr)
    if arr.ndim == 1:
        string = ' '.join(map(str, arr))
        return string

    elif arr.ndim == 2:
        string_list = []
        for arr_line in arr:
            string = ' '.join(map(str, arr_line)) + "\n"
            string_list.append(string)
        if return_type =='str':
            string = str(string_list)[2:-2].replace("', '", "").replace("\\n", "\n")
            return string
        elif return_type =='list':
            return string_list
        else:
            raise ValueError("Only 'str' and 'list' can be selected for return_type")
    else:
        raise ValueError("Only supports arrays of 2 dimensions and below")

def is_empty_directory(directory):
    assert isinstance(directory, Path),"This path is not a directory"
    return len(os.listdir(directory)) == 0


def delete_empty_directories(directory):
    path = Path(directory)
    for dir_to_check in path.rglob('*'):
        if dir_to_check.is_dir() and not any(dir_to_check.iterdir()):
            dir_to_check.rmdir()

def get_sorted_files(directory="."):
    files = [f for f in os.listdir(directory) if os.path.isfile(os.path.join(directory, f))]
    files.sort()
    return files

def get_sorted_directories(directory="."):
    dirs = [f for f in os.listdir(directory) if os.path.isdir(os.path.join(directory, f))]
    dirs.sort()
    return dirs


import numpy as np
import math

def xy2rt(x,y):
    # 球坐标系
    r = np.sqrt(x**2 + y**2)
    theta = np.arctan2(y, x)
    return r, theta


def rt2xy(r, theta):
    # 极坐标系转直角坐标系
    x = r * np.cos(theta)
    y = r * np.sin(theta)
    return x, y

def xyz2rtp(x, y, z):
    # 柱标系
    r = np.sqrt(x**2 + y**2 + z**2)
    phi = np.arccos(z / r)
    theta = np.arctan2(y, x)
    return r, theta, phi


def rtp2xyz(r,theta,phi):
    x = r * np.sin(phi) * np.cos(theta)
    y = r * np.sin(phi) * np.sin(theta)
    z = r * np.cos(phi)
    return x, y, z


import math
import numpy as np

def point_in_polygon(point, polygon):
        # 在圆内就返回1，圆外返回0
        x, y = point
        n = len(polygon)
        count = 0
        for i in range(n):
            x1, y1 = polygon[i]
            x2, y2 = polygon[(i + 1) % n]
            # 判断点是否在多边形顶点处
            if (x == x1 and y == y1) or (x == x2 and y == y2):
                return 1
            if min(y1, y2) < y <= max(y1, y2):
                # 计算交点的横坐标
                x0 = x1 + (y - y1) * (x2 - x1) / (y2 - y1)
                # 如果交点位于射线之右，则计数器加1
                if x0 > x:
                    count += 1     
        return 1 if count % 2 == 1 else 0


from setuptools import setup, find_packages
import os
import sys

try:
    from torch.utils.cpp_extension import BuildExtension, CUDAExtension

    TORCH = True
except:
    TORCH = False

'''
any: 适用于任何平台的通用版本。
manylinux1_x86_64: 适用于符合ManyLinux规范的x86_64 Linux系统。
win_amd64: 适用于64位Windows系统。
macosx_10_9_x86_64: 适用于OS X 10.9及以上版本的x86_64 Mac系统
'''


def make_cuda_ext(name, module, sources):
    # from torch.utils.cpp_extension import BuildExtension, CUDAExtension
    cuda_ext = CUDAExtension(
        name='%s.%s' % (module, name),
        sources=[os.path.join(*module.split('.'), src) for src in sources]
    )
    return cuda_ext


setupinfo = dict(
    name='wata',  # 包名
    version='0.1.4.0',  # 版本
    description="wangtao tools",  # 包简介
    platforms=['Linux','Windows','MacOS'],
    long_description=open('README.md', encoding='utf-8').read(),  # 读取文件中介绍包的详细内容
    include_package_data=True,  # 是否允许上传资源文件
    author='wangtao',  # 作者
    author_email='1083719817@qq.com',  # 作者邮件
    maintainer='wangtao',  # 维护者
    maintainer_email='1083719817@qq.com',  # 维护者邮件
    license='MIT License',  # 协议
    url='',  # github或者自己的网站地址
    packages=find_packages(),  # 包的目录
    package_data={'': ['*.yaml', '*.txt', '*.bin', '*.pcd', '*.png', '*.ui',]},
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    options={
        'bdist_wheel': {
            'python_tag': 'py38',
            'plat_name': 'manylinux1_x86_64',
            'build_number': None,
            'dist_dir': None,
        }
    },
    python_requires='>=3.6',
    install_requires=['numpy',
                    #   'PyQt5',
                    #   'PyOpenGL',
                    #   'pyqtgraph',
                      'python-lzf',
                      'matplotlib',
                        # 'opencv-python==4.1.2.30',
                        # 'opencv-contrib-python',
                        # 'utm',
                      'tqdm', 
                      'utm',
                      'pyyaml', 'vtk', 'scipy',
                    #   'tabulate',
                      'pyquaternion'],
)

if sys.platform.startswith("linux") and TORCH == True:
    setup(
        name=setupinfo['name'],
        version=setupinfo['version'],
        description=setupinfo['description'],
        platforms=setupinfo['platforms'],
        long_description=setupinfo['long_description'],
        include_package_data=setupinfo['include_package_data'],
        author=setupinfo['author'],
        author_email=setupinfo['author_email'],
        maintainer=setupinfo['maintainer'],
        maintainer_email=setupinfo['maintainer_email'],
        license=setupinfo['license'],
        url=setupinfo['url'],
        packages=setupinfo['packages'],
        package_data=setupinfo['package_data'],
        classifiers=setupinfo['classifiers'],
        options=setupinfo['options'],
        python_requires=setupinfo['python_requires'],
        install_requires=setupinfo['install_requires'],
        cmdclass={'build_ext': BuildExtension, },
        ext_modules=[
            make_cuda_ext(
                name='roiaware_pool3d_cuda',
                module='wata.pointcloud.ops.roiaware_pool3d',
                sources=[
                    'src/roiaware_pool3d.cpp',
                    'src/roiaware_pool3d_kernel.cu',
                ]
            ),
        ],
    )

else:
    setup(
        name=setupinfo['name'],
        version=setupinfo['version'],
        description=setupinfo['description'],
        platforms=setupinfo['platforms'],
        long_description=setupinfo['long_description'],
        include_package_data=setupinfo['include_package_data'],
        author=setupinfo['author'],
        author_email=setupinfo['author_email'],
        maintainer=setupinfo['maintainer'],
        maintainer_email=setupinfo['maintainer_email'],
        license=setupinfo['license'],
        url=setupinfo['url'],
        packages=setupinfo['packages'],
        package_data=setupinfo['package_data'],
        classifiers=setupinfo['classifiers'],
        options=setupinfo['options'],
        python_requires=setupinfo['python_requires'],
        install_requires=setupinfo['install_requires'],
        # cmdclass=setupinfo['cmdclass'],
        # ext_modules=setupinfo['ext_modules'],
    )



from wata.image.utils import utils
from pathlib import Path
from typing import Union

class ImageProcess:
    @staticmethod
    def show_img(path:Union[str, Path]):
        '''
        **功能描述**: 展示图像  
        
        Args:
            path: 图像路径  

        Returns:
            无  
        '''
        utils.show_img(path)

    @staticmethod
    def img2video(img_dir, save_path, fps=30):
        '''
        **功能描述**: 将图像连成视频  
        
        Args:
            img_dir: 图像目录的路径  
            save_path: 视频保存地址  
            fps: 视频帧率,默认30  

        Returns:  
            无  
        '''
        utils.images_to_video(img_dir, save_path, fps)

    @staticmethod
    def video2img(path, save_path):
        '''
        **功能描述**: 将视频展开成图像  
        
        Args:
            path: 视频地址  
            save_path: 图像保存地址  

        Returns:
            无  
        '''
        utils.video_to_images(path, save_path)
